package prefuse.data;

import java.io.PrintStream;
import java.util.Iterator;
import prefuse.data.column.Column;
import prefuse.data.event.ColumnListener;
import prefuse.data.event.GraphListener;
import prefuse.data.event.TableListener;
import prefuse.data.expression.Predicate;
import prefuse.data.tuple.CompositeTupleSet;
import prefuse.data.tuple.TableEdge;
import prefuse.data.tuple.TableNode;
import prefuse.data.tuple.TupleManager;
import prefuse.data.tuple.TupleSet;
import prefuse.data.util.Index;
import prefuse.data.util.NeighborIterator;
import prefuse.util.PrefuseConfig;
import prefuse.util.collections.CompositeIntIterator;
import prefuse.util.collections.CompositeIterator;
import prefuse.util.collections.CopyOnWriteArrayList;
import prefuse.util.collections.IntArrayIterator;
import prefuse.util.collections.IntIterator;

public class Graph
  extends CompositeTupleSet
{
  public static final int INEDGES = 0;
  public static final int OUTEDGES = 1;
  public static final int UNDIRECTED = 2;
  public static final String DEFAULT_NODE_KEY = PrefuseConfig.get("data.graph.nodeKey");
  public static final String DEFAULT_SOURCE_KEY = PrefuseConfig.get("data.graph.sourceKey");
  public static final String DEFAULT_TARGET_KEY = PrefuseConfig.get("data.graph.targetKey");
  public static final String NODES = PrefuseConfig.get("data.graph.nodeGroup");
  public static final String EDGES = PrefuseConfig.get("data.graph.edgeGroup");
  protected Table m_links;
  protected TupleManager m_nodeTuples;
  protected TupleManager m_edgeTuples;
  protected boolean m_directed = false;
  protected SpanningTree m_spanning = null;
  protected String m_nkey;
  protected String m_skey;
  protected String m_tkey;
  protected Index m_nidx;
  private Listener m_listener;
  private CopyOnWriteArrayList m_listeners = new CopyOnWriteArrayList();
  protected static final String INDEGREE = "_indegree";
  protected static final String OUTDEGREE = "_outdegree";
  protected static final String INLINKS = "_inlinks";
  protected static final String OUTLINKS = "_outlinks";
  protected static final Schema LINKS_SCHEMA = new Schema();
  
  public Graph()
  {
    this(false);
  }
  
  public Graph(boolean paramBoolean)
  {
    this(new Table(), paramBoolean);
  }
  
  public Graph(Table paramTable, boolean paramBoolean)
  {
    this(paramTable, paramBoolean, DEFAULT_NODE_KEY, DEFAULT_SOURCE_KEY, DEFAULT_TARGET_KEY);
  }
  
  public Graph(Table paramTable, boolean paramBoolean, String paramString1, String paramString2, String paramString3)
  {
    Table localTable = new Table();
    localTable.addColumn(paramString2, Integer.TYPE, new Integer(-1));
    localTable.addColumn(paramString3, Integer.TYPE, new Integer(-1));
    init(paramTable, localTable, paramBoolean, paramString1, paramString2, paramString3);
  }
  
  public Graph(Table paramTable1, Table paramTable2, boolean paramBoolean)
  {
    this(paramTable1, paramTable2, paramBoolean, DEFAULT_NODE_KEY, DEFAULT_SOURCE_KEY, DEFAULT_TARGET_KEY);
  }
  
  public Graph(Table paramTable1, Table paramTable2, boolean paramBoolean, String paramString1, String paramString2)
  {
    init(paramTable1, paramTable2, paramBoolean, DEFAULT_NODE_KEY, paramString1, paramString2);
  }
  
  public Graph(Table paramTable1, Table paramTable2, boolean paramBoolean, String paramString1, String paramString2, String paramString3)
  {
    init(paramTable1, paramTable2, paramBoolean, paramString1, paramString2, paramString3);
  }
  
  protected void init(Table paramTable1, Table paramTable2, boolean paramBoolean, String paramString1, String paramString2, String paramString3)
  {
    if (((paramString1 != null) && (paramTable1.getColumnType(paramString1) != Integer.TYPE)) || (paramTable2.getColumnType(paramString2) != Integer.TYPE) || (paramTable2.getColumnType(paramString3) != Integer.TYPE)) {
      throw new IllegalArgumentException("Incompatible column types for graph keys");
    }
    removeAllSets();
    super.addSet(EDGES, paramTable2);
    super.addSet(NODES, paramTable1);
    this.m_directed = paramBoolean;
    this.m_nkey = paramString1;
    this.m_skey = paramString2;
    this.m_tkey = paramString3;
    if (paramString1 != null)
    {
      paramTable1.index(paramString1);
      this.m_nidx = paramTable1.getIndex(paramString1);
    }
    if (this.m_nodeTuples == null) {
      this.m_nodeTuples = new TupleManager(paramTable1, this, TableNode.class);
    }
    this.m_edgeTuples = new TupleManager(paramTable2, this, TableEdge.class);
    initLinkTable();
    if (this.m_listener == null) {
      this.m_listener = new Listener();
    }
    paramTable1.addTableListener(this.m_listener);
    paramTable2.addTableListener(this.m_listener);
    this.m_listener.setEdgeTable(paramTable2);
  }
  
  public void setTupleManagers(TupleManager paramTupleManager1, TupleManager paramTupleManager2)
  {
    if (!Node.class.isAssignableFrom(paramTupleManager1.getTupleType())) {
      throw new IllegalArgumentException("The provided node TupleManager must generate tuples that implement the Node interface.");
    }
    if (!Edge.class.isAssignableFrom(paramTupleManager2.getTupleType())) {
      throw new IllegalArgumentException("The provided edge TupleManager must generate tuples that implement the Edge interface.");
    }
    this.m_nodeTuples = paramTupleManager1;
    this.m_edgeTuples = paramTupleManager2;
  }
  
  public void dispose()
  {
    getNodeTable().removeTableListener(this.m_listener);
    getEdgeTable().removeTableListener(this.m_listener);
  }
  
  public void setEdgeTable(Table paramTable)
  {
    Table localTable = getEdgeTable();
    localTable.removeTableListener(this.m_listener);
    this.m_edgeTuples.invalidateAll();
    this.m_links.clear();
    init(getNodeTable(), paramTable, this.m_directed, this.m_nkey, this.m_skey, this.m_tkey);
  }
  
  protected void initLinkTable()
  {
    this.m_links = createLinkTable();
    IntIterator localIntIterator = getEdgeTable().rows();
    while (localIntIterator.hasNext()) {
      updateDegrees(localIntIterator.nextInt(), 1);
    }
  }
  
  protected Table createLinkTable()
  {
    return LINKS_SCHEMA.instantiate(getNodeCount());
  }
  
  protected void updateDegrees(int paramInt1, int paramInt2)
  {
    if (!getEdgeTable().isValidRow(paramInt1)) {
      return;
    }
    int i = getSourceNode(paramInt1);
    int j = getTargetNode(paramInt1);
    if ((i < 0) || (j < 0)) {
      return;
    }
    updateDegrees(paramInt1, i, j, paramInt2);
    if (paramInt2 < 0) {
      this.m_edgeTuples.invalidate(paramInt1);
    }
  }
  
  protected void updateDegrees(int paramInt1, int paramInt2, int paramInt3, int paramInt4)
  {
    int i = this.m_links.getInt(paramInt2, "_outdegree");
    int j = this.m_links.getInt(paramInt3, "_indegree");
    if (paramInt4 > 0)
    {
      addLink("_outlinks", i, paramInt2, paramInt1);
      addLink("_inlinks", j, paramInt3, paramInt1);
    }
    else if (paramInt4 < 0)
    {
      remLink("_outlinks", i, paramInt2, paramInt1);
      remLink("_inlinks", j, paramInt3, paramInt1);
    }
    this.m_links.setInt(paramInt2, "_outdegree", i + paramInt4);
    this.m_links.setInt(paramInt3, "_indegree", j + paramInt4);
    this.m_spanning = null;
  }
  
  protected void addLink(String paramString, int paramInt1, int paramInt2, int paramInt3)
  {
    Object localObject = (int[])this.m_links.get(paramInt2, paramString);
    if (localObject == null)
    {
      localObject = new int[] { paramInt3 };
      this.m_links.set(paramInt2, paramString, localObject);
      return;
    }
    if (paramInt1 == localObject.length)
    {
      int[] arrayOfInt = new int[Math.max(3 * localObject.length / 2, paramInt1 + 1)];
      System.arraycopy(localObject, 0, arrayOfInt, 0, localObject.length);
      localObject = arrayOfInt;
      this.m_links.set(paramInt2, paramString, localObject);
    }
    localObject[paramInt1] = paramInt3;
  }
  
  protected boolean remLink(String paramString, int paramInt1, int paramInt2, int paramInt3)
  {
    int[] arrayOfInt = (int[])this.m_links.get(paramInt2, paramString);
    for (int i = 0; i < paramInt1; i++) {
      if (arrayOfInt[i] == paramInt3)
      {
        System.arraycopy(arrayOfInt, i + 1, arrayOfInt, i, paramInt1 - i - 1);
        return true;
      }
    }
    return false;
  }
  
  protected void updateNodeData(int paramInt, boolean paramBoolean)
  {
    if (paramBoolean)
    {
      this.m_links.addRow();
    }
    else
    {
      this.m_nodeTuples.invalidate(paramInt);
      this.m_links.removeRow(paramInt);
    }
  }
  
  public String getNodeKeyField()
  {
    return this.m_nkey;
  }
  
  public String getEdgeSourceField()
  {
    return this.m_skey;
  }
  
  public String getEdgeTargetField()
  {
    return this.m_tkey;
  }
  
  public int getKey(int paramInt)
  {
    return this.m_nkey == null ? paramInt : getNodeTable().getInt(paramInt, this.m_nkey);
  }
  
  public int getNodeIndex(int paramInt)
  {
    if (this.m_nidx == null) {
      return paramInt;
    }
    int i = this.m_nidx.get(paramInt);
    return i < 0 ? -1 : i;
  }
  
  public int addNodeRow()
  {
    return getNodeTable().addRow();
  }
  
  public Node addNode()
  {
    int i = addNodeRow();
    return (Node)this.m_nodeTuples.getTuple(i);
  }
  
  public int addEdge(int paramInt1, int paramInt2)
  {
    int i = getKey(paramInt1);
    int j = getKey(paramInt2);
    Table localTable = getEdgeTable();
    int k = localTable.addRow();
    localTable.setInt(k, this.m_skey, i);
    localTable.setInt(k, this.m_tkey, j);
    return k;
  }
  
  public Edge addEdge(Node paramNode1, Node paramNode2)
  {
    nodeCheck(paramNode1, true);
    nodeCheck(paramNode2, true);
    int i = addEdge(paramNode1.getRow(), paramNode2.getRow());
    return getEdge(i);
  }
  
  public boolean removeNode(int paramInt)
  {
    Table localTable = getNodeTable();
    if (localTable.isValidRow(paramInt))
    {
      int i = getInDegree(paramInt);
      if (i > 0)
      {
        int[] arrayOfInt1 = (int[])this.m_links.get(paramInt, "_inlinks");
        int k = i;
        for (;;)
        {
          k--;
          if (k < 0) {
            break;
          }
          removeEdge(arrayOfInt1[k]);
        }
      }
      int j = getOutDegree(paramInt);
      if (j > 0)
      {
        int[] arrayOfInt2 = (int[])this.m_links.get(paramInt, "_outlinks");
        int m = j;
        for (;;)
        {
          m--;
          if (m < 0) {
            break;
          }
          removeEdge(arrayOfInt2[m]);
        }
      }
    }
    return localTable.removeRow(paramInt);
  }
  
  public boolean removeNode(Node paramNode)
  {
    nodeCheck(paramNode, true);
    return removeNode(paramNode.getRow());
  }
  
  public boolean removeEdge(int paramInt)
  {
    return getEdgeTable().removeRow(paramInt);
  }
  
  public boolean removeEdge(Edge paramEdge)
  {
    edgeCheck(paramEdge, true);
    return removeEdge(paramEdge.getRow());
  }
  
  protected void clearEdges()
  {
    getEdgeTable().clear();
  }
  
  protected boolean nodeCheck(Node paramNode, boolean paramBoolean)
  {
    if (!paramNode.isValid())
    {
      if (paramBoolean) {
        throw new IllegalArgumentException("Node must be valid.");
      }
      return false;
    }
    Graph localGraph = paramNode.getGraph();
    if ((localGraph != this) && (localGraph.m_spanning != this))
    {
      if (paramBoolean) {
        throw new IllegalArgumentException("Node must be part of this Graph.");
      }
      return false;
    }
    return true;
  }
  
  public TupleSet getNodes()
  {
    return getSet(NODES);
  }
  
  public Table getNodeTable()
  {
    return (Table)getSet(NODES);
  }
  
  public int getNodeCount()
  {
    return getNodeTable().getRowCount();
  }
  
  public Node getNode(int paramInt)
  {
    if (paramInt < 0) {
      System.err.println("catch");
    }
    return (Node)this.m_nodeTuples.getTuple(paramInt);
  }
  
  public Node getNodeFromKey(int paramInt)
  {
    int i = getNodeIndex(paramInt);
    return i < 0 ? null : getNode(i);
  }
  
  public int getInDegree(int paramInt)
  {
    return this.m_links.getInt(paramInt, "_indegree");
  }
  
  public int getInDegree(Node paramNode)
  {
    nodeCheck(paramNode, true);
    return getInDegree(paramNode.getRow());
  }
  
  public int getOutDegree(int paramInt)
  {
    return this.m_links.getInt(paramInt, "_outdegree");
  }
  
  public int getOutDegree(Node paramNode)
  {
    nodeCheck(paramNode, true);
    return getOutDegree(paramNode.getRow());
  }
  
  public int getDegree(int paramInt)
  {
    return getInDegree(paramInt) + getOutDegree(paramInt);
  }
  
  public int getDegree(Node paramNode)
  {
    nodeCheck(paramNode, true);
    return getDegree(paramNode.getRow());
  }
  
  public boolean isDirected()
  {
    return this.m_directed;
  }
  
  protected boolean edgeCheck(Edge paramEdge, boolean paramBoolean)
  {
    if (!paramEdge.isValid())
    {
      if (paramBoolean) {
        throw new IllegalArgumentException("Edge must be valid.");
      }
      return false;
    }
    if (paramEdge.getGraph() != this)
    {
      if (paramBoolean) {
        throw new IllegalArgumentException("Edge must be part of this Graph.");
      }
      return false;
    }
    return true;
  }
  
  public TupleSet getEdges()
  {
    return getSet(EDGES);
  }
  
  public Table getEdgeTable()
  {
    return (Table)getSet(EDGES);
  }
  
  public int getEdgeCount()
  {
    return getEdgeTable().getRowCount();
  }
  
  public Edge getEdge(int paramInt)
  {
    return paramInt < 0 ? null : (Edge)this.m_edgeTuples.getTuple(paramInt);
  }
  
  public int getEdge(int paramInt1, int paramInt2)
  {
    int i = getOutDegree(paramInt1);
    if (i > 0)
    {
      int[] arrayOfInt = (int[])this.m_links.get(paramInt1, "_outlinks");
      for (int j = 0; j < i; j++) {
        if (getTargetNode(arrayOfInt[j]) == paramInt2) {
          return arrayOfInt[j];
        }
      }
    }
    return -1;
  }
  
  public Edge getEdge(Node paramNode1, Node paramNode2)
  {
    nodeCheck(paramNode1, true);
    nodeCheck(paramNode2, true);
    return getEdge(getEdge(paramNode1.getRow(), paramNode2.getRow()));
  }
  
  public int getSourceNode(int paramInt)
  {
    return getNodeIndex(getEdgeTable().getInt(paramInt, this.m_skey));
  }
  
  public Node getSourceNode(Edge paramEdge)
  {
    edgeCheck(paramEdge, true);
    return getNode(getSourceNode(paramEdge.getRow()));
  }
  
  public int getTargetNode(int paramInt)
  {
    return getNodeIndex(getEdgeTable().getInt(paramInt, this.m_tkey));
  }
  
  public Node getTargetNode(Edge paramEdge)
  {
    edgeCheck(paramEdge, true);
    return getNode(getTargetNode(paramEdge.getRow()));
  }
  
  public int getAdjacentNode(int paramInt1, int paramInt2)
  {
    int i = getSourceNode(paramInt1);
    int j = getTargetNode(paramInt1);
    if (i == paramInt2) {
      return j;
    }
    if (j == paramInt2) {
      return i;
    }
    throw new IllegalArgumentException("Edge is not incident on the input node.");
  }
  
  public Node getAdjacentNode(Edge paramEdge, Node paramNode)
  {
    edgeCheck(paramEdge, true);
    nodeCheck(paramNode, true);
    return getNode(getAdjacentNode(paramEdge.getRow(), paramNode.getRow()));
  }
  
  public IntIterator nodeRows()
  {
    return getNodeTable().rows();
  }
  
  public IntIterator edgeRows()
  {
    return getEdgeTable().rows();
  }
  
  public IntIterator edgeRows(int paramInt)
  {
    return edgeRows(paramInt, 2);
  }
  
  public IntIterator edgeRows(int paramInt1, int paramInt2)
  {
    int[] arrayOfInt;
    if (paramInt2 == 1)
    {
      arrayOfInt = (int[])this.m_links.get(paramInt1, "_outlinks");
      return new IntArrayIterator(arrayOfInt, 0, getOutDegree(paramInt1));
    }
    if (paramInt2 == 0)
    {
      arrayOfInt = (int[])this.m_links.get(paramInt1, "_inlinks");
      return new IntArrayIterator(arrayOfInt, 0, getInDegree(paramInt1));
    }
    if (paramInt2 == 2) {
      return new CompositeIntIterator(edgeRows(paramInt1, 1), edgeRows(paramInt1, 0));
    }
    throw new IllegalArgumentException("Unrecognized edge type: " + paramInt2 + ". Type should be one of Graph.OUTEDGES, " + "Graoh.INEDGES, or Graph.ALL");
  }
  
  public IntIterator inEdgeRows(int paramInt)
  {
    return edgeRows(paramInt, 0);
  }
  
  public IntIterator outEdgeRows(int paramInt)
  {
    return edgeRows(paramInt, 1);
  }
  
  public Iterator nodes()
  {
    return this.m_nodeTuples.iterator(nodeRows());
  }
  
  public Iterator neighbors(Node paramNode)
  {
    return new NeighborIterator(paramNode, edges(paramNode));
  }
  
  public Iterator inNeighbors(Node paramNode)
  {
    return new NeighborIterator(paramNode, inEdges(paramNode));
  }
  
  public Iterator outNeighbors(Node paramNode)
  {
    return new NeighborIterator(paramNode, outEdges(paramNode));
  }
  
  public Iterator edges()
  {
    return this.m_edgeTuples.iterator(edgeRows());
  }
  
  public Iterator edges(Node paramNode)
  {
    nodeCheck(paramNode, true);
    return this.m_edgeTuples.iterator(edgeRows(paramNode.getRow(), 2));
  }
  
  public Iterator inEdges(Node paramNode)
  {
    nodeCheck(paramNode, true);
    return this.m_edgeTuples.iterator(inEdgeRows(paramNode.getRow()));
  }
  
  public Iterator outEdges(Node paramNode)
  {
    nodeCheck(paramNode, true);
    return this.m_edgeTuples.iterator(outEdgeRows(paramNode.getRow()));
  }
  
  public void clear()
  {
    this.m_nodeTuples.invalidateAll();
    this.m_edgeTuples.invalidateAll();
    super.clear();
    this.m_links.clear();
  }
  
  public boolean removeTuple(Tuple paramTuple)
  {
    if ((paramTuple instanceof Node)) {
      return removeNode((Node)paramTuple);
    }
    if ((paramTuple instanceof Edge)) {
      return removeEdge((Edge)paramTuple);
    }
    throw new IllegalArgumentException("Input tuple must be part of this graph");
  }
  
  public Iterator tuples(Predicate paramPredicate)
  {
    if (paramPredicate == null) {
      return tuples();
    }
    return new CompositeIterator(this.m_edgeTuples.iterator(getEdgeTable().rows(paramPredicate)), this.m_nodeTuples.iterator(getNodeTable().rows(paramPredicate)));
  }
  
  public Iterator tuples()
  {
    return new CompositeIterator(edges(), nodes());
  }
  
  public Tree getSpanningTree()
  {
    if (this.m_spanning == null) {
      return getSpanningTree((Node)nodes().next());
    }
    return this.m_spanning;
  }
  
  public Tree getSpanningTree(Node paramNode)
  {
    nodeCheck(paramNode, true);
    if (this.m_spanning == null) {
      this.m_spanning = new SpanningTree(this, paramNode);
    } else if (this.m_spanning.getRoot() != paramNode) {
      this.m_spanning.buildSpanningTree(paramNode);
    }
    return this.m_spanning;
  }
  
  public void clearSpanningTree()
  {
    this.m_spanning = null;
  }
  
  public void addGraphModelListener(GraphListener paramGraphListener)
  {
    if (!this.m_listeners.contains(paramGraphListener)) {
      this.m_listeners.add(paramGraphListener);
    }
  }
  
  public void removeGraphModelListener(GraphListener paramGraphListener)
  {
    this.m_listeners.remove(paramGraphListener);
  }
  
  protected void fireGraphEvent(Table paramTable, int paramInt1, int paramInt2, int paramInt3, int paramInt4)
  {
    String str = paramTable == getNodeTable() ? NODES : EDGES;
    if (paramInt4 != 0) {
      fireTupleEvent(paramTable, paramInt1, paramInt2, paramInt4);
    }
    if (!this.m_listeners.isEmpty())
    {
      Object[] arrayOfObject = this.m_listeners.getArray();
      for (int i = 0; i < arrayOfObject.length; i++) {
        ((GraphListener)arrayOfObject[i]).graphChanged(this, str, paramInt1, paramInt2, paramInt3, paramInt4);
      }
    }
  }
  
  static
  {
    Integer localInteger = new Integer(0);
    LINKS_SCHEMA.addColumn("_indegree", Integer.TYPE, localInteger);
    LINKS_SCHEMA.addColumn("_outdegree", Integer.TYPE, localInteger);
    LINKS_SCHEMA.addColumn("_inlinks", int[].class);
    LINKS_SCHEMA.addColumn("_outlinks", int[].class);
  }
  
  protected class Listener
    implements TableListener, ColumnListener
  {
    private Table m_edges;
    private Column m_scol;
    private Column m_tcol;
    private int m_sidx;
    private int m_tidx;
    
    protected Listener() {}
    
    public void setEdgeTable(Table paramTable)
    {
      if (this.m_scol != null) {
        this.m_scol.removeColumnListener(this);
      }
      if (this.m_tcol != null) {
        this.m_tcol.removeColumnListener(this);
      }
      this.m_scol = (this.m_tcol = null);
      this.m_sidx = (this.m_tidx = -1);
      this.m_edges = paramTable;
      if (this.m_edges != null)
      {
        this.m_sidx = paramTable.getColumnNumber(Graph.this.m_skey);
        this.m_tidx = paramTable.getColumnNumber(Graph.this.m_tkey);
        this.m_scol = paramTable.getColumn(this.m_sidx);
        this.m_tcol = paramTable.getColumn(this.m_tidx);
        this.m_scol.addColumnListener(this);
        this.m_tcol.addColumnListener(this);
      }
    }
    
    public void tableChanged(Table paramTable, int paramInt1, int paramInt2, int paramInt3, int paramInt4)
    {
      if (!Graph.this.containsSet(paramTable)) {
        throw new IllegalStateException("Graph shouldn't be listening to an unrelated table");
      }
      if (paramInt4 != 0)
      {
        boolean bool;
        int i;
        if (paramTable == Graph.this.getNodeTable())
        {
          Graph.this.m_nodeTuples.setMaximumRow(Graph.this.getNodeCount());
          if (paramInt3 == -1)
          {
            bool = paramInt4 == 1;
            for (i = paramInt1; i <= paramInt2; i++) {
              Graph.this.updateNodeData(i, bool);
            }
          }
        }
        else
        {
          Graph.this.m_edgeTuples.setMaximumRow(Graph.this.getEdgeCount());
          if (paramInt3 == -1)
          {
            bool = paramInt4 == 1;
            for (i = paramInt1; i <= paramInt2; i++) {
              Graph.this.updateDegrees(paramInt1, bool ? 1 : -1);
            }
          }
        }
        Graph.this.m_spanning = null;
      }
      Graph.this.fireGraphEvent(paramTable, paramInt1, paramInt2, paramInt3, paramInt4);
    }
    
    public void columnChanged(Column paramColumn, int paramInt1, int paramInt2)
    {
      if ((paramColumn == this.m_scol) || (paramColumn == this.m_tcol))
      {
        int i = paramColumn == this.m_scol ? 1 : 0;
        int j = this.m_edges.getTableRow(paramInt1, i != 0 ? this.m_sidx : this.m_tidx);
        if (j == -1) {
          return;
        }
        int k = Graph.this.getSourceNode(j);
        int m = Graph.this.getTargetNode(j);
        int n = Graph.this.getNodeIndex(paramInt2);
        if ((n > -1) && (((i != 0) && (m > -1)) || ((i == 0) && (k > -1)))) {
          Graph.this.updateDegrees(j, i != 0 ? n : k, i != 0 ? m : n, -1);
        }
        if ((k > -1) && (m > -1)) {
          Graph.this.updateDegrees(j, k, m, 1);
        }
      }
      else
      {
        throw new IllegalStateException();
      }
    }
    
    public void columnChanged(Column paramColumn, int paramInt1, int paramInt2, int paramInt3)
    {
      throw new IllegalStateException();
    }
    
    public void columnChanged(Column paramColumn, int paramInt, long paramLong)
    {
      throw new IllegalStateException();
    }
    
    public void columnChanged(Column paramColumn, int paramInt, float paramFloat)
    {
      throw new IllegalStateException();
    }
    
    public void columnChanged(Column paramColumn, int paramInt, double paramDouble)
    {
      throw new IllegalStateException();
    }
    
    public void columnChanged(Column paramColumn, int paramInt, boolean paramBoolean)
    {
      throw new IllegalStateException();
    }
    
    public void columnChanged(Column paramColumn, int paramInt, Object paramObject)
    {
      throw new IllegalStateException();
    }
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\data\Graph.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */