package prefuse.data.util;

import java.util.Comparator;
import java.util.Iterator;
import prefuse.data.Table;
import prefuse.data.expression.AndPredicate;
import prefuse.data.expression.ColumnExpression;
import prefuse.data.expression.ComparisonPredicate;
import prefuse.data.expression.Expression;
import prefuse.data.expression.ExpressionAnalyzer;
import prefuse.data.expression.NotPredicate;
import prefuse.data.expression.OrPredicate;
import prefuse.data.expression.Predicate;
import prefuse.data.expression.RangePredicate;
import prefuse.data.tuple.TupleSet;
import prefuse.util.PrefuseConfig;
import prefuse.util.collections.CompositeIntIterator;
import prefuse.util.collections.IntIterator;

public class FilterIteratorFactory
{
  private static final int OPTIMIZATION_THRESHOLD = PrefuseConfig.getInt("data.filter.optimizeThreshold");
  
  public static Iterator tuples(TupleSet paramTupleSet, Predicate paramPredicate)
  {
    if (paramPredicate == null) {
      return paramTupleSet.tuples();
    }
    Object localObject = null;
    if ((paramTupleSet instanceof Table))
    {
      Table localTable = (Table)paramTupleSet;
      IntIterator localIntIterator = getOptimizedIterator(localTable, paramPredicate);
      if (localIntIterator != null) {
        localObject = localTable.tuples(localIntIterator);
      }
    }
    if (localObject == null) {
      localObject = new FilterIterator(paramTupleSet.tuples(), paramPredicate);
    }
    return (Iterator)localObject;
  }
  
  public static IntIterator rows(Table paramTable, Predicate paramPredicate)
  {
    Object localObject = null;
    localObject = getOptimizedIterator(paramTable, paramPredicate);
    if (localObject == null) {
      localObject = new FilterRowIterator(paramTable.rows(), paramTable, paramPredicate);
    }
    return (IntIterator)localObject;
  }
  
  protected static IntIterator getOptimizedIterator(Table paramTable, Predicate paramPredicate)
  {
    if (paramTable.getRowCount() < OPTIMIZATION_THRESHOLD) {
      return null;
    }
    if ((paramPredicate instanceof ColumnExpression)) {
      return getColumnIterator(paramTable, ((ColumnExpression)paramPredicate).getColumnName(), true);
    }
    if ((paramPredicate instanceof NotPredicate))
    {
      Predicate localPredicate = ((NotPredicate)paramPredicate).getPredicate();
      if ((localPredicate instanceof ColumnExpression)) {
        return getColumnIterator(paramTable, ((ColumnExpression)localPredicate).getColumnName(), false);
      }
    }
    else
    {
      if ((paramPredicate instanceof AndPredicate)) {
        return getAndIterator(paramTable, (AndPredicate)paramPredicate);
      }
      if ((paramPredicate instanceof OrPredicate)) {
        return getOrIterator(paramTable, (OrPredicate)paramPredicate);
      }
      if ((paramPredicate instanceof ComparisonPredicate)) {
        return getComparisonIterator(paramTable, (ComparisonPredicate)paramPredicate);
      }
      if ((paramPredicate instanceof RangePredicate)) {
        return getRangeIterator(paramTable, (RangePredicate)paramPredicate);
      }
    }
    return null;
  }
  
  protected static IntIterator getColumnIterator(Table paramTable, String paramString, boolean paramBoolean)
  {
    if (paramTable.getColumnType(paramString) != Boolean.TYPE) {
      return null;
    }
    Index localIndex = paramTable.getIndex(paramString);
    if (localIndex == null) {
      return null;
    }
    return localIndex.rows(paramBoolean);
  }
  
  protected static IntIterator getOrIterator(Table paramTable, OrPredicate paramOrPredicate)
  {
    int i = paramOrPredicate.size();
    if (i > 1)
    {
      IntIterator[] arrayOfIntIterator = new IntIterator[i];
      for (int j = 0; j < arrayOfIntIterator.length; j++)
      {
        arrayOfIntIterator[j] = getOptimizedIterator(paramTable, paramOrPredicate.get(j));
        if (arrayOfIntIterator[j] == null) {
          return null;
        }
      }
      return new UniqueRowIterator(new CompositeIntIterator(arrayOfIntIterator));
    }
    if (i == 1) {
      return getOptimizedIterator(paramTable, paramOrPredicate.get(0));
    }
    return null;
  }
  
  protected static IntIterator getAndIterator(Table paramTable, AndPredicate paramAndPredicate)
  {
    IntIterator localIntIterator = null;
    Predicate localPredicate = null;
    int i = paramAndPredicate.size();
    for (;;)
    {
      i--;
      if (i >= 0)
      {
        localPredicate = paramAndPredicate.get(i);
        if ((localIntIterator = getOptimizedIterator(paramTable, localPredicate)) != null) {
          break;
        }
      }
    }
    if (localIntIterator == null) {
      return null;
    }
    if (paramAndPredicate.size() == 1) {
      return localIntIterator;
    }
    return new FilterRowIterator(localIntIterator, paramTable, paramAndPredicate.getSubPredicate(localPredicate));
  }
  
  protected static IntIterator getComparisonIterator(Table paramTable, ComparisonPredicate paramComparisonPredicate)
  {
    Expression localExpression1 = paramComparisonPredicate.getLeftExpression();
    Expression localExpression2 = paramComparisonPredicate.getRightExpression();
    int i = paramComparisonPredicate.getOperation();
    if (i == 3) {
      return null;
    }
    ColumnExpression localColumnExpression;
    Expression localExpression3;
    if (((localExpression1 instanceof ColumnExpression)) && (!ExpressionAnalyzer.hasDependency(localExpression2)))
    {
      localColumnExpression = (ColumnExpression)localExpression1;
      localExpression3 = localExpression2;
    }
    else if (((localExpression2 instanceof ColumnExpression)) && (!ExpressionAnalyzer.hasDependency(localExpression1)))
    {
      localColumnExpression = (ColumnExpression)localExpression2;
      localExpression3 = localExpression1;
    }
    else
    {
      return null;
    }
    Comparator localComparator = paramComparisonPredicate.getComparator();
    Index localIndex = paramTable.getIndex(localColumnExpression.getColumnName());
    if ((localIndex == null) || (!localComparator.equals(localIndex.getComparator()))) {
      return null;
    }
    Class localClass = localExpression3.getClass();
    if (localClass == Integer.TYPE)
    {
      int j = localExpression3.getInt(null);
      switch (i)
      {
      case 0: 
        return localIndex.rows(Integer.MIN_VALUE, j, 41);
      case 1: 
        return localIndex.rows(j, Integer.MAX_VALUE, 38);
      case 2: 
        return localIndex.rows(j, j, 42);
      case 4: 
        return localIndex.rows(Integer.MIN_VALUE, j, 42);
      case 5: 
        return localIndex.rows(j, Integer.MAX_VALUE, 42);
      }
      throw new IllegalStateException();
    }
    if (localClass == Long.TYPE)
    {
      long l = localExpression3.getLong(null);
      switch (i)
      {
      case 0: 
        return localIndex.rows(Long.MIN_VALUE, l, 41);
      case 1: 
        return localIndex.rows(l, Long.MAX_VALUE, 38);
      case 2: 
        return localIndex.rows(l, l, 42);
      case 4: 
        return localIndex.rows(Long.MIN_VALUE, l, 42);
      case 5: 
        return localIndex.rows(l, Long.MAX_VALUE, 42);
      }
      throw new IllegalStateException();
    }
    if (localClass == Float.TYPE)
    {
      float f = localExpression3.getFloat(null);
      switch (i)
      {
      case 0: 
        return localIndex.rows(Float.MIN_VALUE, f, 41);
      case 1: 
        return localIndex.rows(f, Float.MAX_VALUE, 38);
      case 2: 
        return localIndex.rows(f, f, 42);
      case 4: 
        return localIndex.rows(Float.MIN_VALUE, f, 42);
      case 5: 
        return localIndex.rows(f, Float.MAX_VALUE, 42);
      }
      throw new IllegalStateException();
    }
    if (localClass == Double.TYPE)
    {
      double d = localExpression3.getDouble(null);
      switch (i)
      {
      case 0: 
        return localIndex.rows(Double.MIN_VALUE, d, 41);
      case 1: 
        return localIndex.rows(d, Double.MAX_VALUE, 38);
      case 2: 
        return localIndex.rows(d, d, 42);
      case 4: 
        return localIndex.rows(Double.MIN_VALUE, d, 42);
      case 5: 
        return localIndex.rows(d, Double.MAX_VALUE, 42);
      }
      throw new IllegalStateException();
    }
    Object localObject = localExpression3.get(null);
    switch (i)
    {
    case 0: 
      return localIndex.rows(null, localObject, 41);
    case 1: 
      return localIndex.rows(localObject, null, 38);
    case 2: 
      return localIndex.rows(localObject, localObject, 42);
    case 4: 
      return localIndex.rows(null, localObject, 42);
    case 5: 
      return localIndex.rows(localObject, null, 42);
    }
    throw new IllegalStateException();
  }
  
  protected static IntIterator getRangeIterator(Table paramTable, RangePredicate paramRangePredicate)
  {
    if ((!(paramRangePredicate.getMiddleExpression() instanceof ColumnExpression)) || (ExpressionAnalyzer.hasDependency(paramRangePredicate.getLeftExpression())) || (ExpressionAnalyzer.hasDependency(paramRangePredicate.getRightExpression()))) {
      return null;
    }
    ColumnExpression localColumnExpression = (ColumnExpression)paramRangePredicate.getMiddleExpression();
    Expression localExpression1 = paramRangePredicate.getLeftExpression();
    Expression localExpression2 = paramRangePredicate.getRightExpression();
    Comparator localComparator = paramRangePredicate.getComparator();
    Index localIndex = paramTable.getIndex(localColumnExpression.getColumnName());
    if ((localIndex == null) || (!localComparator.equals(localIndex.getComparator()))) {
      return null;
    }
    int i = paramRangePredicate.getOperation();
    Class localClass = paramTable.getColumnType(localColumnExpression.getColumnName());
    int j;
    switch (i)
    {
    case 0: 
      j = 42;
      break;
    case 1: 
      j = 41;
      break;
    case 2: 
      j = 38;
      break;
    case 3: 
      j = 37;
      break;
    default: 
      throw new IllegalStateException();
    }
    if (localClass == Integer.TYPE) {
      return localIndex.rows(localExpression1.getInt(null), localExpression2.getInt(null), j);
    }
    if (localClass == Long.TYPE) {
      return localIndex.rows(localExpression1.getLong(null), localExpression2.getLong(null), j);
    }
    if (localClass == Float.TYPE) {
      return localIndex.rows(localExpression1.getFloat(null), localExpression2.getFloat(null), j);
    }
    if (localClass == Double.TYPE) {
      return localIndex.rows(localExpression1.getDouble(null), localExpression2.getDouble(null), j);
    }
    return localIndex.rows(localExpression1.get(null), localExpression2.get(null), j);
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\data\util\FilterIteratorFactory.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */